function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var FormControl = function FormControl(state, reducer) {
  var _this = this;

  this.subscription = [];

  this.dispatch = function (action) {
    var old = _this.formState;
    _this.formState = _this.reducer(_this.formState, action);

    if (old !== _this.formState) {
      _this.notifySubscriber();
    }
  };

  this.addSubscription = function (sub) {
    _this.subscription = [].concat(_this.subscription, [sub]);
    return function () {
      _this.subscription = _this.subscription.filter(function (i) {
        return i !== sub;
      });
    };
  };

  this.removeSubscription = function (sub) {
    _this.subscription = _this.subscription.filter(function (i) {
      return i !== sub;
    });
  };

  this.getState = function () {
    return _this.formState;
  };

  this.notifySubscriber = function () {
    _this.subscription.forEach(function (sub) {
      sub(_this.formState);
    });
  };

  this.formState = {
    values: state,
    errors: Object.keys(state).reduce(function (acc, key) {
      var _extends2;

      return _extends({}, acc, (_extends2 = {}, _extends2[key] = '', _extends2));
    }, {}),
    touched: Object.keys(state).reduce(function (acc, key) {
      var _extends3;

      return _extends({}, acc, (_extends3 = {}, _extends3[key] = false, _extends3));
    }, {})
  };
  this.reducer = reducer;
};

var createFormControl = function createFormControl(initialForm, reducer) {
  return new FormControl(initialForm, reducer);
};

function createAction(actionName) {
  return function () {
    var result = function result(payload) {
      return {
        type: actionName,
        payload: payload
      };
    };

    result.getType = function () {
      return actionName;
    };

    return result;
  };
}
function getActionType(action) {
  return action.getType();
}
function shallowEqual(objA, objB) {
  if (objA === objB) {
    return true;
  }

  if (!objA || !objB) {
    return false;
  }

  var aKeys = Object.keys(objA);
  var bKeys = Object.keys(objB);
  var len = aKeys.length;

  if (bKeys.length !== len) {
    return false;
  }

  for (var i = 0; i < len; i++) {
    var key = aKeys[i];

    if (objA[key] !== objB[key] || !Object.prototype.hasOwnProperty.call(objB, key)) {
      return false;
    }
  }

  return true;
}

var FormoContext = React.createContext(null);
var FormoProvider = FormoContext.Provider;
function useFormoContext() {
  var formoValues = React.useContext(FormoContext);

  if (!formoValues) {
    throw new Error('Please wrap in Formo component');
  }

  return formoValues;
}

var changeValue = createAction('CHANGE_VALUE')();
var changeTouched = createAction('CHANGE_TOUCHED')();
var setValues = createAction('SET_VALUES')();
var setErrors = createAction('SET_ERRORS')();
var setTouched = createAction('SET_TOUCHED')();
var resetValues = createAction('RESET_VALUES')();

function Formo(_ref) {
  var initialValue = _ref.initialValue,
      children = _ref.children,
      onSubmit = _ref.onSubmit,
      enableReinitialize = _ref.enableReinitialize;
  var initialRef = React.useRef();
  var reducers = React.useCallback(function (state, action) {
    switch (action.type) {
      case getActionType(changeValue):
        {
          var _extends2;

          return _extends({}, state, {
            values: _extends({}, state.values, (_extends2 = {}, _extends2[action.payload.key] = action.payload.value, _extends2))
          });
        }

      case getActionType(changeTouched):
        {
          var _extends3;

          return _extends({}, state, {
            touched: _extends({}, state.touched, (_extends3 = {}, _extends3[action.payload.key] = action.payload.value, _extends3))
          });
        }

      case getActionType(setValues):
        {
          return _extends({}, state, {
            values: _extends({}, state.values, action.payload)
          });
        }

      case getActionType(setErrors):
        {
          return _extends({}, state, {
            errors: _extends({}, state.errors, action.payload)
          });
        }

      case getActionType(setTouched):
        {
          return _extends({}, state, {
            touched: _extends({}, state.touched, action.payload)
          });
        }

      case getActionType(resetValues):
        {
          return _extends({}, state, initialRef.current);
        }

      default:
        {
          return state;
        }
    }
  }, []);

  var _useState = React.useState(createFormControl(initialValue, reducers)),
      formControl = _useState[0];

  React.useEffect(function () {
    initialRef.current = formControl.getState();
  }, []);
  var formValue = React.useMemo(function () {
    return {
      getState: formControl.getState,
      addSubscription: formControl.addSubscription,
      removeSubscription: formControl.removeSubscription,
      dispatch: formControl.dispatch,
      form: {
        submit: onSubmit
      }
    };
  }, [onSubmit, formControl]);
  React.useEffect(function () {
    if (enableReinitialize) {
      formControl.dispatch(setValues(initialValue));
      initialRef.current = formControl.getState();
    }
  }, [initialValue, enableReinitialize]);
  return React__default.createElement(FormoProvider, {
    value: formValue
  }, children);
}

function Form(_ref) {
  var onSubmit = _ref.onSubmit,
      children = _ref.children;

  var _useFormoContext = useFormoContext(),
      getState = _useFormoContext.getState,
      form = _useFormoContext.form,
      dispatch = _useFormoContext.dispatch;

  var handleSubmit = React.useCallback(function (e) {
    e.preventDefault();
    e.stopPropagation();
    var formState = getState();
    var formHelper = {
      reset: function reset() {
        dispatch(resetValues());
      },
      errors: formState.errors,
      touched: formState.touched,
      values: formState.values,
      setErrors: function setErrors$1(values) {
        dispatch(setErrors(values));
      },
      setTouched: function setTouched(values) {
        dispatch(setValues(values));
      },
      setValues: function setValues$1(values) {
        dispatch(setValues(values));
      }
    };
    var values = getState().values;

    if (onSubmit) {
      onSubmit(values, formHelper);
    } else {
      form.submit(values, formHelper);
    }
  }, []);
  var handleReset = React.useCallback(function (e) {
    e.preventDefault();
    e.stopPropagation();
    dispatch(resetValues());
  }, []);
  return React__default.createElement("form", {
    onSubmit: handleSubmit,
    onReset: handleReset
  }, children);
}

var useRerender = function useRerender() {
  var _useState = React.useState(0),
      setC = _useState[1];

  return React.useCallback(function () {
    return setC(function (p) {
      return p + 1;
    });
  }, []);
};

function useField(name) {
  var context = useFormoContext();
  var rerender = useRerender();
  var selector = React.useRef(function (state) {
    return {
      value: state.values[name],
      touched: state.touched[name],
      error: state.errors[name]
    };
  }).current;
  var values = React.useRef(selector(context.getState()));
  React.useEffect(function () {
    return context.addSubscription(function (value) {
      var newValue = selector(value);

      if (!shallowEqual(newValue, values.current)) {
        values.current = newValue;
        rerender();
      }
    });
  }, [name]);
  var setFieldValue = React.useCallback(function (value) {
    return context.dispatch(changeValue({
      key: name,
      value: value
    }));
  }, [name]);
  var field = {
    name: name,
    value: values.current.value,
    onChange: function onChange(e) {
      context.dispatch(changeValue({
        key: name,
        value: e.target.value
      }));
    },
    onBlur: function onBlur() {
      context.dispatch(changeTouched({
        key: name,
        value: true
      }));
    }
  };
  var helper = React.useMemo(function () {
    return {
      setFieldValue: setFieldValue
    };
  }, [setFieldValue]);
  return [field, helper];
}

function Field(_ref) {
  var component = _ref.component,
      name = _ref.name,
      children = _ref.children,
      render = _ref.render,
      other = _objectWithoutPropertiesLoose(_ref, ["component", "name", "children", "render"]);

  var _useField = useField(name),
      field = _useField[0],
      helper = _useField[1];

  if (component) {
    var C = component;
    return React__default.createElement(C, Object.assign({}, field, other));
  }

  var renderProps = render || children;

  if (renderProps) {
    return renderProps(field, helper);
  }

  return React__default.createElement("input", Object.assign({}, field, other));
}

function useValues(names) {
  var context = useFormoContext();
  var rerender = useRerender();
  var fieldNames = React.useMemo(function () {
    return names;
  }, [names]);
  var selector = React.useRef(function (state) {
    return (fieldNames === null || fieldNames === void 0 ? void 0 : fieldNames.reduce(function (acc, name) {
      var _extends2;

      return _extends({}, acc, (_extends2 = {}, _extends2[name] = state.values[name], _extends2));
    }, {})) || state.values;
  }).current;
  var values = React.useRef(selector(context.getState()));
  React.useEffect(function () {
    return context.addSubscription(function (value) {
      var newValue = selector(value);

      if (!shallowEqual(newValue, values.current)) {
        values.current = newValue;
        rerender();
      }
    });
  }, [name]);
  return values.current;
}

exports.Field = Field;
exports.Form = Form;
exports.Formo = Formo;
exports.createAction = createAction;
exports.getActionType = getActionType;
exports.shallowEqual = shallowEqual;
exports.useField = useField;
exports.useValues = useValues;
//# sourceMappingURL=index.js.map
